#!/usr/bin/env python

'''Show current RTP sessions'''

## mangle path depending on version (for binary package). also add modules subdir
import sys, os
version = '%d.%d' % sys.version_info[0:2]
sys.path[1:1] = [os.path.normpath((sys.path[0] or '.') + '/' + version)]
sys.path[0:0] = [os.path.normpath((sys.path[0] or '.') + '/modules')]
del os

def normalizeBytes(bytes):
    mb = bytes/1024/1024.0
    kb = bytes/1024.0
    if mb >= 0.95:
        return "%.2fM" % mb
    elif kb >= 1:
        return "%.2fk" %kb
    else:
        return "%d" % bytes

def normalizeTraffic(traffic):
    mb = traffic/1024/1024.0
    kb = traffic/1024.0
    if mb >= 0.95:
        return "%.2fMbps" % mb
    elif kb >= 1:
        return "%.2fkbps" %kb
    else:
        return "%dbps" % traffic

def normalizeTime(period):
    min, sec = divmod(period, 60)
    h, min   = divmod(min, 60)
    if h >= 1:
        return '%dh%02d\'%02d"' % (h, min, sec)
    else:
        return '%d\'%02d"' % (min, sec)


if __name__ == '__main__':
    import socket
    import sys, errno
    import re, time
    from optparse import OptionParser
    from options import ExtendedOptions
    from formatting import Table

    parser = OptionParser(option_class=ExtendedOptions, version="%prog 1.0")
    parser.add_option("--proxy", dest="address", type="controlport",
                      default='/var/run/mediaproxy.sock',
                      help="mediaproxy's control port (/var/run/mediaproxy.sock)", 
                      metavar="Host|File")
    #parser.add_option("--active", action="store_true", dest="active", default=0,
    #                  help="show active connections only")
    (options, args) = parser.parse_args()
    if type(options.address) == type(()):
        type = socket.AF_INET
    else:
        type =  socket.AF_UNIX
    s = socket.socket(type, socket.SOCK_STREAM)
    try:
        s.connect(options.address)
    except socket.error, why:
        if why[0] == errno.ENOENT:
            print >>sys.stderr, "RTP Proxy's control socket doesn't exist (%s)" % options.address
        else:
            print >>sys.stderr, "Cannot connect to RTP Proxy's control socket: %s" % str(why[1])
        sys.exit(1)
    try:
        s.send('status\n')
        result = ''
        while 1:
            try:
                data = s.recv(1024)
                if not data:
                    break
                result += data
            except socket.error, why:
                print >>sys.stderr, "Cannot read result from MediaProxy: %s" % str(why[1])
                sys.exit(1)
    finally:
        s.close()

    header = ['Caller', '', 'Via', '', 'Called', '', 'Status', 'Duration', 'Codec', 'Type',
              'Traffic']
    output = Table(header)
    output.header.bodySeparator = '-'
    output.header.separators = [' ', '   ', ' ', '   ', ' ', '  ', '  ', '  ', '  ', '  ', '']
    output.separators =        [':', ' - ', ':', ' - ', ':', '  ', '  ', '  ', '  ', '  ', '']
    output.alignments = ['right', 'left', 'right', 'left', 'right', 'left', 'left',
                         'right', 'left', 'left', 'left']

    traffic = {'in1': 0, 'in2': 0, 'out': 0}
    
    lines = result.strip().splitlines()
    for l in lines:
        m = re.search(r'''(?P<id>[^\s]+)                      # session id
                          \s+ 
                          (?P<adr1>[^:\s]+):(?P<port1>[^\s]+) # 1st address:port
                          \s+
                          (?P<adr>[^:\s]+):(?P<port>[^\s]+)   # Proxy address:port
                          \s+
                          (?P<adr2>[^:\s]+):(?P<port2>[^\s]+) # 2nd address:port
                          \s+
                          (?P<status>[^\s]+)                  # Status
                          \s+
                          (?P<duration>[^\s]+)                # Call duration
                          \s+
                          (?P<idle>[^\s]+)                    # Idle time
                          \s+
                          (?P<bytes1>\d+)                     # Bytes from peer1
                          /
                          (?P<bytes>\d+)                      # Bytes relayed
                          /
                          (?P<bytes2>\d+)                     # Bytes from peer2
                          \s+
                          (?P<rtpCodec>[^\s]+)                # RTP codec type
                          \s+
                          (?P<streamType>[^\s]+)              # RTP stream type (audio/video)
                       ''', l.strip(), re.VERBOSE)
        if not m:
            print >>sys.stderr, "Invalid response line: %s" % l.strip()
            continue
        else:
            duration = float(m.group('duration'))
            idle = int(float(m.group('idle')))
            status = m.group('status')
            if status == 'idle':
                status += ' (%02d)' % idle
            bytes  = long(m.group('bytes'))
            bytes1 = long(m.group('bytes1'))
            bytes2 = long(m.group('bytes2'))
            if status == 'active':
                traffic['in1'] += bytes1 * 8 / duration
                traffic['in2'] += bytes2 * 8 / duration
                traffic['out'] += bytes  * 8 / duration
            nbytes = "%s/%s/%s" % (normalizeBytes(bytes1), normalizeBytes(bytes),
                                   normalizeBytes(bytes2))
            session = [m.group('adr1'), m.group('port1'),
                       m.group('adr'),  m.group('port'),
                       m.group('adr2'), m.group('port2'),
                       status, normalizeTime(duration), 
                       m.group('rtpCodec'), m.group('streamType'), 
                       nbytes]
            output.append(session)

    if not output:
        print 'No RTP sessions'
        sys.exit(0)

    count = len(output)

    print ''
    output.show()
    print ''
    print 'Total traffic:  %s/%s/%s (in1/in2/out)' % \
          (normalizeTraffic(traffic['in1']),
           normalizeTraffic(traffic['in2']),
           normalizeTraffic(traffic['out']))
    #print ''
    print 'Session count:  %s' % count
    print ''

